/* global fis */
var config = require('./config');
var spawn = require('cross-spawn');
var fs = require('fs');
var path = require('path');
var extendsMethd = {
    web: 1,
    check: 1,
    doc: 1,
    sync: 1,
    user: 1,
    search: 1,
};
exports.name = 'npm';
exports.desc = 'npm packager';
exports.options = {
    '-h, --help': 'print this help message',
    '--files': 'some options.',
    '-r, --registry [registry]': 'registry url, default is ' + config.cnpmRegistry,
    '-w, --registryweb [registryweb]': 'web url, default is ' + config.cnpmHost,
    '--disturl [disturl]': 'dist url for node-gyp, default is ' + config.disturl,
    '-c, --cache [cache]': 'cache folder, default is ' + config.cache,
    '-u, --userconfig [userconfig]': 'userconfig file, default is ' + config.userconfig,
    '-y, --yes': 'yes all confirm'
};

exports.run = function (argv, cli) {
    var CWD = process.cwd();
    argv.userconfig = argv.userconfig || config.userconfig;
    if (argv.h || argv.help) {
        return cli.help(exports.name, exports.options);
    }
    if (!argv.registry) {
        // try to use registry in uerconfig
        argv.registry = getDefaultRegistry(argv.userconfig);
        function getDefaultRegistry(userconfig) {
            if (argv.userconfig !== 'none' && fs.existsSync(argv.userconfig)) {
                var content = fs.readFileSync(argv.userconfig, 'utf8');
                // registry = {registry-url}
                var m = /^registry\s*=\s*(.+)$/m.exec(content);
                if (m) {
                    return m[1];
                }
            }
            return config.cnpmRegistry;
        }
    }
    if (!argv.disturl) {
        var isIOJS = process.execPath.indexOf('iojs') >= 0;
        argv.disturl = isIOJS ? config.iojsDisturl : config.disturl;
    }
    if (argv.disturl === 'none') {
        delete argv.disturl;
    }
    if (argv.userconfig === 'none') {
        delete argv.userconfig;
    }
    argv.registryweb = argv.registryweb || config.cnpmHost;
    var rawArgs = argv._;
    var action = rawArgs[1];
    var args = rawArgs.slice(1);
    var packageUrl = fis.project.getProjectPath() + '/package.json';
    argv.packageUrl = packageUrl;
    if (extendsMethd[action]) {
        return require('./lib/' + action)(argv);
    }
    
    function getGitUrl(name) {
        var host = 'git+http://git.superboss.cc/';

        var map = {
            'tatami': 'zhengchao/raycloud'
        };
        if (map[name]) {
            return host + map[name] + '.git';
        }
        return '';
    }
    function getNpm(url) {
        var name = url;
        var version = '';
        var _tmp = url.split('@');
        name = _tmp[0];
        if (_tmp[1]) {
            version = _tmp[1];
        }

        return {
            ver: version,
            name: name
        };
    }
    // 可以通过 argv 知道命令行中有哪些参数以及是什么值。
    if (action === 'install') {
        if (args.length === 1) {
            var packageJson;
            try {
                packageJson = fis.util.readJSON(packageUrl);
            } catch (e) {
                fis.log.warn(e);
            }

            if (packageJson && packageJson.dependencies) {
                fis.util.map(packageJson.dependencies, function (name, version) {
                    var gitUrl = getGitUrl(name);
                    if (gitUrl) {
                        var ver = version.split('#')[1];
                        version = gitUrl;
                        if(ver){
                            version += '#'+ver;
                        }
                    }
                });
                fis.util.write(packageUrl,JSON.stringify(packageJson));
            }
        } else {
            var pack = getNpm(args[1]);
            var _url = getGitUrl(pack.name);
            if (_url) {
                args[1] = _url;
                if (pack.ver) {
                    args[1] += '#' + pack.ver;
                }
            }
        }
    }
    
    if (argv.userconfig && !fs.existsSync(argv.userconfig)) {
        fs.writeFileSync(argv.userconfig, 'email =\n');
    }

    var nodeModulesDir = path.join(__dirname, 'node_modules');
    var nodegyp = path.join(nodeModulesDir, 'node-gyp', 'bin', 'node-gyp.js');
    args.unshift('--node-gyp=' + nodegyp);
    args.unshift('--registry=' + argv.registry);
    args.unshift('--cache=' + argv.cache);
    if (argv.disturl) {
        args.unshift('--disturl=' + argv.disturl);
    }
    if (argv.userconfig) {
        args.unshift('--userconfig=' + argv.userconfig);
    }

    var npmBin;

    try {
        npmBin = require.resolve('npm');
        npmBin = path.join(npmBin, '..', '..', '..', '.bin', 'npm');
    } catch (_) {
        npmBin = path.join(path.dirname(process.execPath), 'npm');
    }

    var env = {
        NVM_NODEJS_ORG_MIRROR: config.mirrorsUrl + '/node',
        NVM_IOJS_ORG_MIRROR: config.mirrorsUrl + '/iojs',
        PHANTOMJS_CDNURL: config.mirrorsUrl + '/phantomjs',
        CHROMEDRIVER_CDNURL: 'http://oss.npm.taobao.org/dist/chromedriver',
        SELENIUM_CDNURL: config.mirrorsUrl + '/selenium',
        ELECTRON_MIRROR: config.mirrorsUrl + '/electron/',
        SASS_BINARY_SITE: config.mirrorsUrl + '/node-sass',
    };

    for (var k in process.env) {
        env[k] = process.env[k];
    }

    var npm = spawn(npmBin, args, {
        env: env,
        cwd: CWD,
        stdio: [
            process.stdin,
            process.stdout,
            process.stderr,
        ]
    });

    npm.on('exit', function (code, signal) {
        process.exit(code);
    });
};