/* global fis */
/*!
 * cnpm - bin/cnpm-check
 *
 * Copyright(c) cnpmjs.org and other contributors.
 * MIT Licensed
 *
 * Authors:
 *  fengmk2 <fengmk2@gmail.com> (http://fengmk2.github.com)
 */

'use strict';

/**
 * Module dependencies.
 */

var path = require('path');
var urllib = require('urllib');
var config = require('../config');

module.export = function (argv) {
    var pkg;
    try {
        pkg = fis.util.readJSON(argv.packageUrl);
    } catch (e) {
        fis.log.warn(e);
    }
    var names = Object.keys(pkg.dependencies);
    var next = function () {
        var name = names.shift();
        if (!name) {
            return process.exit(0);
        }
        var url = config.cnpmRegistry + '/' + name + '/latest';
        urllib.request(url, { headers: { 'user-agent': 'cnpm-check' } },
            function (err, result) {
                var errmsg = '';
                if (err) {
                    errmsg = err.message;
                } else {
                    try {
                        result = JSON.parse(result);
                    } catch (e) {
                        errmsg = 'parse origin package info error. ' + err.message;
                        result = {};
                    }
                }
                if (errmsg) {
                    console.log('[' + name.red + '] check version error: ' + err.message.red);
                    return next();
                }

                var localVersion = String(pkg.dependencies[name] || '*');
                var localVersionCheck = localVersion.replace(/^[^\.\d]+/g, '');
                var remoteVersion = result && result.version || 'unknow';
                if (remoteVersion !== localVersionCheck) {
                    console.log('[' + name.yellow + '] new version: ' + remoteVersion.green + ', local version: ' + localVersion.yellow);
                }
                next();
            });
    };

    next();
}